
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "libconf.h"
#include "getstream.h"

static struct config_s	*config;
static struct adapter_s	*adapter;
static struct stream_s	*stream;
static struct channel_s	*channel;
static struct sap_s	*sap;

static int cf_adapter_start(char *t, char *v) {
	struct adapter_s	*a;
	a=calloc(1, sizeof(struct adapter_s));
	a->no=strtol(v, NULL, 10);
	a->next=adapter;
	adapter=a;
	config->adapter=a;
	return 0;
}

static int cf_sap_scope(char *t, char *v) { sap->scope=v; return 0; }
static int cf_sap_sap_group(char *t, char *v) { sap->group=v; return 0; }
static int cf_sap_sap_port(char *t, char *v) { sap->port=strtol(v, NULL, 10); return 0; }
static int cf_sap_ttl(char *t, char *v) { sap->ttl=strtol(v, NULL, 10); return 0; }
static int cf_sap_announce_host(char *t, char *v) { sap->announcehost=v; return 0; }
static int cf_sap_announce_port(char *t, char *v) { sap->announceport=strtol(v, NULL, 10); return 0; }

static int cf_stream_remoteport(char *t, char *v) { stream->remoteport=strtol(v, NULL, 10); return 0; }
static int cf_stream_remoteaddr(char *t, char *v) { stream->remoteaddr=v; return 0; }
static int cf_stream_rtpport(char *t, char *v) { stream->rtpport=strtol(v, NULL, 10); return 0; }
static int cf_stream_rtcpport(char *t, char *v) { stream->rtcpport=strtol(v, NULL, 10); return 0; }
static int cf_stream_ttl(char *t, char *v) { stream->ttl=strtol(v, NULL, 10); return 0; }
static int cf_stream_url(char *t, char *v) { stream->url=v; return 0; }
static int cf_http_port(char *t, char *v) { config->http_port=strtol(v, NULL, 10); return 0; }

static int cf_sap_start(char *t, char *v) {
	sap=calloc(1, sizeof(struct sap_s));
	stream->sap=sap;
	sap->stream=stream;

	/* Default values */
	sap->ttl=15;
	return 0;
}

static int cf_channel_start(char *t, char *v) {
	struct channel_s	*c;
	c=calloc(1, sizeof(struct channel_s));
	c->id=strtol(v, NULL, 16);
	c->next=adapter->channel;
	adapter->channel=c;
	channel=c;
	return 0;
}

static int cf_stream_start(char *t, char *v, int stype) {
	struct stream_s		*s;
	s=calloc(1, sizeof(struct stream_s));
	s->next=channel->stream;
	channel->stream=s;
	s->channel=channel;
	s->type=stype;
	stream=s;

	/* Default values */
	s->ttl=15;
	return 0;
}

static int cf_stream_udp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_UDP); }
static int cf_stream_rtp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_RTP); }
static int cf_stream_rtcp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_RTCP); }
static int cf_stream_http_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_HTTP); }

struct lc_ventry conf_sap[] = {
	{ "scope", 0, 1, LCV_STRING, 0, NULL, cf_sap_scope },
	{ "sap-group", 0, 1, LCV_IPV4ADDR, 0, NULL, cf_sap_sap_group },
	{ "sap-port", 0, 1, LCV_NUM, 0, NULL, cf_sap_sap_port },
	{ "announce-host", 0, 1, LCV_STRING, 0, NULL, cf_sap_announce_host },
	{ "announce-port", 0, 1, LCV_NUM, 0, NULL, cf_sap_announce_port },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_sap_ttl },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_udp[] = {
	{ "remote-address", 1, 1, LCV_IPV4ADDR, 0, NULL, cf_stream_remoteaddr },
	{ "remote-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_remoteport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_stream_ttl },
	{ NULL, 0, 0, 0, 0, NULL, NULL },
};

struct lc_ventry conf_stream_rtp[] = {
	{ "remote-address", 1, 1, LCV_IPV4ADDR, 0, NULL, cf_stream_remoteaddr },
	{ "remote-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_remoteport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_stream_ttl },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_rtcp[] = {
	{ "rtp-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_rtpport },
	{ "rtcp-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_rtcpport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_http[] = {
	{ "url", 1, 1, LCV_STRING, 0, NULL, cf_stream_url },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_channel[] = {
	{ "stream-udp", 0, 0, LCV_NONE, 0, conf_stream_udp, cf_stream_udp_start },
	{ "stream-rtp", 0, 0, LCV_NONE, 0, conf_stream_rtp, cf_stream_rtp_start },
	{ "stream-rtcp", 0, 0, LCV_NONE, 0, conf_stream_rtcp, cf_stream_rtcp_start },
	{ "stream-http", 0, 0, LCV_NONE, 0, conf_stream_http, cf_stream_http_start },
	{ NULL, 0, 0, 0, 0, NULL },
};

static struct lc_ventry conf_adapter[] = {
	{ "channel", 0, 0, LCV_HEX, LCO_UNIQ, conf_channel, cf_channel_start },
	{ NULL, 0, 0, 0, 0, NULL },
};


static struct lc_ventry conf_http[] = {
	{ "port", 1, 1, LCV_NUM, 0, NULL, cf_http_port },
};

static struct lc_ventry conf_main[] = {
	{ "adapter", 1, 1, LCV_NUM, LCO_UNIQ, conf_adapter, cf_adapter_start },
	{ "http", 1, 1, LCV_NONE, 0, conf_http, NULL },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct config_s *readconfig(char *filename) {
	int			cfd;
	struct stat		sb;
	char			*ctext;
	struct lc_centry	*c;

	/* Base for config storage */
	config=calloc(1, sizeof(struct config_s));

	cfd=open(filename, O_RDONLY);
	if (cfd<0)
		return NULL;

	if (fstat(cfd, &sb)) {
		close(cfd);
		return NULL;
	}

	ctext=mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, cfd, 0);

	c=libconf_parse(ctext, sb.st_size);

	munmap(config, sb.st_size);
	close(cfd);

	if (!c)
		return NULL;

	if (!libconf_validate(c, conf_main)) {
		libconf_free(c);
		return NULL;
	}

	return config;
}
