#include <stdio.h>
#include <malloc.h>
#include <pwd.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/param.h>

#include "dvbrc.h"

#if 0
/*
 * LNB ID 0 TYPE 1  LOF1 9750000 LOF2 10600000 SLOF 11700000 DISEQCNR 0
 *   SAT ID 192 NAME "ASTRA 1B, 1C, 1E, " LNBID 0 FMIN 10700000 FMAX 12700000
 *     TRANSPONDER ID 0014  SATID 0192 TYPE 0 FREQ 11837000 POL H SRATE 27500000 FEC 3/4
 *       CHANNEL ID fd NAME "WDR FERNSEHEN" SATID 192 TPID 14 SID 6dcf TYPE 0 
 */

static struct lnb_s lnb_0 = {
	.id = 0,
	.type = 0,
	.lof1 = 9750000,
	.lof2 = 10600000,
	.slof = 11700000,
};

static struct satellite_s satellite_192 = {
	.id = 0x192,
	.name = "ASTRA 1B, 1C, 1E, ",
	.fmin = 10700000,
	.fmax = 12700000,
	.lnb = &lnb_0,
};

static struct transponder_s transponder_0014 = {
	.id = 0x14,
	.type = 0,
	.freq = 11837000,
	.pol = 'H',
	.srate = 27500000,
	.satellite = &satellite_192,
};

static struct channel_s channel_102 = {
	.id = 0x102,
	.name = "Das Erste",
	.sid = 0x6dca,
	.type = 0,
	.transponder = &transponder_0014,
};

static struct channel_s	channel_fd = {
	.next = &channel_102,
	.id = 0xfd,
	.name = "WDR FERNSEHEN",
	.sid = 0x6dcf,
	.type = 0,
	.transponder = &transponder_0014
};

static struct dvbrc_s dvbrc = {
	.channel = &channel_fd
};

#endif

struct dvbrc_s	*dvbrc=NULL;

struct dvbrc_channel_s *get_channel_by_id(unsigned int cid) {
	struct dvbrc_channel_s	*c;

	if (!dvbrc)
		return NULL;

	c=dvbrc->channel;

	while(c) {
		if (c->id == cid)
			return c;
		c=c->next;
	}

	return NULL;
}

extern FILE *yyin;

int read_dvbrc(void ) {
	struct passwd *p=getpwuid(geteuid());
	char fname[PATH_MAX+1];

	snprintf(fname, PATH_MAX, "%s/.dvbrc", 
	 	(p->pw_dir) ? p->pw_dir : "");

	dvbrc=calloc(1, sizeof(struct dvbrc_s));
	yyin=fopen(fname, "r");
	if (!yyin) {
		printf("No .dvbrc found\n");
		return 0;
	}

	yyparse();

	return 1;
}
