%{
#include <malloc.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "dvbrc.h"

struct dvbrc_channel_s 		*cc;
struct dvbrc_transponder_s	*ct;
struct dvbrc_satellite_s	*cs;
struct dvbrc_lnb_s		*cl;
%}

%union {
	char	*s;
	int	i;
}

%token T_LNB
%token T_ID
%token T_TYPE
%token T_LOF1
%token T_LOF2
%token T_SLOF
%token T_DISEQCNR
%token T_SAT
%token T_NAME
%token T_LNBID
%token T_FMIN
%token T_FMAX
%token T_TRANSPONDER
%token T_SATID
%token T_FREQ
%token T_POL
%token T_SRATE
%token T_FEC
%token T_CHANNEL
%token T_TPID
%token T_TTPID
%token T_SID
%token <s> T_FECS

%token <s> T_NUM
%token <s> T_STRING

%%
input:
	| input line
;

line:	'\n'
	| exp '\n'
;

exp:	lnb
	| sat
	| transponder
	| channel
;

lnb:	T_LNB T_ID T_NUM T_TYPE T_NUM T_LOF1 T_NUM T_LOF2 T_NUM T_SLOF T_NUM T_DISEQCNR T_NUM
	{
		struct dvbrc_lnb_s		*l;
		l=calloc(1, sizeof(struct dvbrc_lnb_s));
		l->id=strtol($3, NULL, 16);
		l->type=strtol($5, NULL, 10);
		l->lof1=strtol($7, NULL, 10);
		l->lof2=strtol($9, NULL, 10);
		l->slof=strtol($11, NULL, 10);
		l->diseqcnr=strtol($13, NULL, 10);

		cl=l;
	}
;

sat:	T_SAT T_ID T_NUM T_NAME T_STRING T_LNBID T_NUM T_FMIN T_NUM T_FMAX T_NUM
	{
		struct dvbrc_satellite_s	*s;

		if (!cl) {
			printf("Need LNB for satellite\n");
			exit(-1);
		}

		s=calloc(1, sizeof(struct dvbrc_satellite_s));
		s->id=strtol($3, NULL, 16);
		s->name=$5;
		s->lnbid=strtol($7, NULL, 16);
		s->fmin=strtol($9, NULL, 10);
		s->fmax=strtol($11, NULL, 10);

		s->lnb=cl;

		cs=s;
	}
;

hv:	'H'	{ ct->pol='H'; }
	| 'V'	{ ct->pol='V'; }
;

transponderstart: T_TRANSPONDER T_ID T_NUM
	{
		struct dvbrc_transponder_s	*t;

		if (!cs) {
			printf("Need satellite for transponder\n");
			exit(-1);
		}
		t=calloc(1, sizeof(struct dvbrc_transponder_s));
		t->id=strtol($3, NULL, 16);
		t->satellite=cs;
		ct=t;
	}
;

transponder: transponderstart T_SATID T_NUM T_TYPE T_NUM T_FREQ T_NUM T_POL hv T_SRATE T_NUM T_FEC T_FECS
	{
		ct->satid=strtol($3, NULL, 16);
		ct->type=strtol($5, NULL, 10);
		ct->freq=strtol($7, NULL, 10);
		ct->srate=strtol($11, NULL, 10);
		ct->fec=$13;
	}
;

channelname:
	| T_NAME T_STRING
	{
		cc->name=$2;
	}
;

ttpid:
	| T_TTPID T_NUM
	{
		cc->ttpid=strtol((char *) $2, NULL, 16);
	}
;
channelstart: T_CHANNEL T_ID T_NUM
	{
		struct dvbrc_channel_s	*c;
		if (!ct) {
			fprintf(stderr, "Need transponder before channel\n");
			exit(-1);
		}
		c=calloc(1, sizeof(struct dvbrc_channel_s));
		c->id=strtol($3, NULL, 16);
		c->transponder=ct;

		/* Chain chanels */
		c->next=cc;
		cc=c;
		dvbrc->channel=c;
	}
;

channel: channelstart channelname T_SATID T_NUM T_TPID T_NUM T_SID T_NUM T_TYPE T_NUM ttpid
	{
		cc->sid=strtol($8, NULL, 16);
		cc->type=strtol($10, NULL, 10);
	}
;

