
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <event.h>

#include "getstream.h"
#include "dvbrc.h"
#include "sap.h"
#include "stream.h"
#include "config.h"
#include "libhttp.h"

int	debuglvl=0;

static void usage(void ) {
	fprintf(stderr, "-c <config file> -d \n");
	exit(-1);
}

struct http_server	*hserver;

int main(int argc, char **argv) {
	extern char		*optarg;
	char			ch;
	struct adapter_s	*a;
	struct config_s		*config=NULL;

	while((ch=getopt(argc, argv, "c:d")) != -1) {
		switch(ch) {
			case 'c':
				config=readconfig(optarg);
				if (!config)
					exit(1);
				break;
			case 'd':
				debuglvl++;
				break;
			default:
				usage();
				break;
		}
	}
	if (!config)
		exit(-1);

	if (!read_dvbrc()) {
		exit(-1);
	}

	/* Find the adapters transponder from channels */
	for(a=config->adapter;a;a=a->next) {
		struct dvbrc_transponder_s	*t=NULL;
		struct channel_s		*c;

		printf("adapter %d\n", a->no);

		for(c=a->channel;c;c=c->next) {
			struct dvbrc_channel_s	*dc;
			dc=get_channel_by_id(c->id);
			if (!dc) {
				fprintf(stderr, "Unknown channel 0x%x\n", c->id);
				exit(1);
			}

			c->dvbchannel=dc;

			if (!t)
				t=dc->transponder;

			if (t != dc->transponder) {
				fprintf(stderr, "Channels from different transponders\n");
				exit(-1);
			}
		}

		if (!t) {
			fprintf(stderr, "Oops - No transponder - No channel ?\n");
			exit(1);
		}

		/* Store transponder into adapter */
		a->transponder=t;
	}

	/* Initialize libevent */
	event_init();

	if (config->http_port)
		hserver=http_init(config->http_port);

	for(a=config->adapter;a;a=a->next) {
		struct channel_s	*c;

		for(c=a->channel;c;c=c->next) {
			stream_init(c->stream);
			sap_init(c->stream);
		}

		/* Tune to the one and only transponder */
		fe_tune(a);
		/* Initialize demux0 dvr0 and co */
		demux_init(a);
	}

	/* Pigs can fly */
	event_dispatch();

	return 0;
}
